// STB_VOL 0.0.1 2010:08:10 06:08:57$
// Copyright (c) 2010 solgracia
// Released under the GNU General Public License
// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// The STB_VOL plots volume against volume averages.
    /// Reproduces the logic described by Lorn at http://www.mypivots.com/
    /// http://www.mypivots.com/board/topic/5997/8/es-short-term-trading-8-5-2010
	/// </summary>
    [Description("The STB_VOL plots volume against volume averages.")]
	public class STB_VOL : Indicator
	{
		#region Variables
        private int depth = 5;          // number of days back for avg
        private int sets = 1;           // excludes current day from avg
        #endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
            Add(new Plot(new Pen(Color.Blue, 2), PlotStyle.Bar, "STB_VOL_DATA")); // 0
            Add(new Plot(Color.DarkOrange, "STB_VOL_AVG")); // 1
            Add(new Line(Color.DarkGray, 0, "Zero line"));
            
			Overlay				= false;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
            if (Bars == null) return;

            DateTime startTime = Bars.Get(0).Time;
            DateTime suppTime = startTime.AddDays(depth);
            DateTime barTime = Bars.Get(CurrentBar).Time; 
            if (barTime < suppTime) return;

            double[] arrayVols = new double[depth];

            int mb = Bars.GetBar(barTime);      // 

            DateTime t = DateTime.Today;
            int barNbr = 0;
            int j = 0;
            for (int i = sets; i < depth + sets; ++i)
            {
                while (Bars.GetBar(barTime.AddDays(-i - j)) == barNbr)  j++;    // jump w/e
                t = barTime.AddDays(-i - j);    // i working days back
                barNbr = Bars.GetBar(t);        // get barNbr for i working day back

                int barIdx = mb - barNbr;       // calculate barIdx
                double v = Volume[barIdx];      // volume on barIdx = current bar - barNbr
                arrayVols[i - sets] = v;
            }

            double mean = 0;
            foreach (double average in arrayVols) mean += average;
            mean /= arrayVols.Length;
            double volAvg = mean;

            double volData = Volume[0];
 
            Values[0].Set(volData);
            Values[1].Set(volAvg);
        }

		#region Properties
        /// <summary>
        /// </summary>
        [Description("Look days back for avg")]
        [GridCategory("Parameters")]
        public int Depth
        {
            get { return depth; }
            set { depth = Math.Max(1, value); }
        }
        /// <summary>
        /// </summary>
        [Description("Exclude last from avg ")]
        [GridCategory("Parameters")]
        public int Sets
        {
            get { return sets; }
            set { sets = value; }
        }
        #endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private STB_VOL[] cacheSTB_VOL = null;

        private static STB_VOL checkSTB_VOL = new STB_VOL();

        /// <summary>
        /// The STB_VOL (Volume Moving Average) plots an exponential moving average (EMA) of volume.
        /// </summary>
        /// <returns></returns>
        public STB_VOL STB_VOL(int depth)
        {
            return STB_VOL(Input, depth);
        }

        /// <summary>
        /// The STB_VOL (Volume Moving Average) plots an exponential moving average (EMA) of volume.
        /// </summary>
        /// <returns></returns>
        public STB_VOL STB_VOL(Data.IDataSeries input, int depth)
        {
            if (cacheSTB_VOL != null)
                for (int idx = 0; idx < cacheSTB_VOL.Length; idx++)
                    if (cacheSTB_VOL[idx].Depth == depth && cacheSTB_VOL[idx].EqualsInput(input))
                        return cacheSTB_VOL[idx];

            lock (checkSTB_VOL)
            {
                checkSTB_VOL.Depth = depth;
                depth = checkSTB_VOL.Depth;

                if (cacheSTB_VOL != null)
                    for (int idx = 0; idx < cacheSTB_VOL.Length; idx++)
                        if (cacheSTB_VOL[idx].Depth == depth && cacheSTB_VOL[idx].EqualsInput(input))
                            return cacheSTB_VOL[idx];

                STB_VOL indicator = new STB_VOL();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Depth = depth;
                Indicators.Add(indicator);
                indicator.SetUp();

                STB_VOL[] tmp = new STB_VOL[cacheSTB_VOL == null ? 1 : cacheSTB_VOL.Length + 1];
                if (cacheSTB_VOL != null)
                    cacheSTB_VOL.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSTB_VOL = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The STB_VOL (Volume Moving Average) plots an exponential moving average (EMA) of volume.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_VOL STB_VOL(int depth)
        {
            return _indicator.STB_VOL(Input, depth);
        }

        /// <summary>
        /// The STB_VOL (Volume Moving Average) plots an exponential moving average (EMA) of volume.
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_VOL STB_VOL(Data.IDataSeries input, int depth)
        {
            return _indicator.STB_VOL(input, depth);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The STB_VOL (Volume Moving Average) plots an exponential moving average (EMA) of volume.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.STB_VOL STB_VOL(int depth)
        {
            return _indicator.STB_VOL(Input, depth);
        }

        /// <summary>
        /// The STB_VOL (Volume Moving Average) plots an exponential moving average (EMA) of volume.
        /// </summary>
        /// <returns></returns>
        public Indicator.STB_VOL STB_VOL(Data.IDataSeries input, int depth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.STB_VOL(input, depth);
        }
    }
}
#endregion
